/*
 * Copyright (C) 2025, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: qiqi49 <qiqi@kylinos.cn>
 *
 */

/* Add shadow for a round button:
Button {
    id: roundButton
    width: 100
    height: 100
    background.radius: 50
}
DtDropShadow {
    anchors.fill: roundButton
    shadowRadius: 10
    z: -1
    shadowData: ShadowData {
        xOffset: 0
        yOffset: 0
        blurRadius: 50
        spread: 0
        shadowColor: Qt.rgba(0 / 255, 0 / 255, 0 / 255, 1)
        childShadow: ShadowData {
            xOffset: 0
            yOffset: 12
            blurRadius: 25
            spread: 0
            shadowColor: "blue"
        }
    }
}
 */
import QtQuick 2.15
import org.ukui.quick.items 1.0
import org.ukui.quick.platform 1.0

Item {
    id: root
    property var shadowData
    property int shadowSize: 0
    property int shadowRadius: 0
    property var activeShadows: []

    Component.onCompleted: {
        updateShadow();
    }

    onShadowDataChanged: {
        updateShadow();
    }

    Component {
        id: shadowCompontent
        ShadowedRectangle {
            anchors.fill: root
            color: "transparent"
            radius: shadowRadius
            property var shadowRectData

            shadow {
                size: shadowRectData ? shadowRectData.blurRadius + shadowRectData.spread : 0
                color: shadowRectData ? shadowRectData.shadowColor : "black"
                xOffset: shadowRectData ? shadowRectData.xOffset : 0
                yOffset: shadowRectData ? shadowRectData.yOffset : 0
            }
        }
    }

    function updateShadow() {
        clearShadow();

        let currentShadow = shadowData;
        let maxSize = 0;
        while (currentShadow) {
            let instance = shadowCompontent.createObject(root, {shadowRectData: currentShadow});
            if (instance) {
                activeShadows.push(instance);
            }

            let offset = Math.max(currentShadow.xOffset, currentShadow.yOffset);
            maxSize = Math.max(maxSize, (offset + currentShadow.blurRadius + currentShadow.spread));

            if (currentShadow.isMultiShadow) {
                currentShadow = currentShadow.childShadow;
            } else {
                currentShadow = null;
            }
        }
        shadowSize = maxSize;
    }
    function clearShadow() {
        activeShadows.forEach(obj => {
            obj.destroy()
        } );
        activeShadows = [];
    }
}
